from pathlib import Path

import pytest

from az_container_helper.keyvault import KeyVault

test_dir = Path(__file__).parent.absolute()
secrets_path = test_dir / "resources" / ".secrets"

KV_URL = "https://test-keyvault.vault.azure.net"

def test_keyvault_with_secrets():
    kv = KeyVault(KV_URL, secrets_path=secrets_path)

    secret_name = "TEST_SECRET"  #noqa: S105
    secret = kv.get_secret(secret_name)
    assert secret.get_secret_value() == "123456" #noqa: S105

    secret_name = "VERY_SECRET_SECRET" #noqa: S105
    secret = kv.get_secret(secret_name)
    assert secret.get_secret_value() == "abcdefgh" #noqa: S105


def test_keyvault_without_secrets():
    with pytest.raises(KeyError):
        kv = KeyVault(KV_URL)
        secret_name = "TEST_SECRET" #noqa: S105
        secret = kv.get_secret(secret_name)
        assert secret.get_secret_value() == "123456" #noqa: S105


import os

from az_container_helper import get_credentials


def test_get_credentials_with_access_token():
    os.environ["ACCESS_TOKEN"] = "test_token" #noqa: S105
    assert get_credentials().get_token().token == "test_token"   #noqa: S105



import os

from azure.core.credentials import AccessToken
from azure.identity import DefaultAzureCredential

# Taken from https://github.com/Azure/azure-sdk-for-python/blob/bb35e8719972a81a07c7359e729f17e2cc0dae40/sdk/identity/azure-identity/samples/custom_credentials.py#L15
class StaticTokenCredential(object):
    """Authenticates with a previously-acquired access token

    Note that an access token is valid only for certain resources and eventually expires. This credential is therefore
    quite limited. An application using it must ensure the token is valid and contains all claims required by any
    service client given an instance of this credential.
    """

    def __init__(self, access_token):
        if isinstance(access_token, AccessToken):
            self._token = access_token
        else:
            # setting expires_on in the past causes Azure SDK clients to call get_token every time they need a token
            self._token = AccessToken(token=access_token, expires_on=0)

    def get_token(self, *scopes, claims = None, tenant_id= None, **kwargs):
        """get_token is the only method a credential must implement"""
        return self._token


def get_credentials():
    """
    Gets a credential to use with Azure services.

    It will first try to read an existing access token from the ACCESS_TOKEN env variable.
    If no existing token is found it will fall back to standard DefaultAzureCredential workflow.

    Returns:
        A credential to use with Azure services.
    """
    if "ACCESS_TOKEN" in os.environ:
        return StaticTokenCredential(os.environ["ACCESS_TOKEN"])
    else:
        return DefaultAzureCredential()


import dotenv
from azure.core.exceptions import HttpResponseError
from azure.keyvault.secrets import SecretClient

from az_container_helper.utils import get_credentials


class Secret:
    """Class to store secret values
    It will prevent the secret value from being printed or logged accidentally.
    """

    def __init__(self, secret_value):
        self._secret_value = secret_value

    def __str__(self):
        return "********" if self._secret_value else "None"

    def get_secret_value(self):
        return self._secret_value


class KeyVault:
    """KeyVault wrapper class to conveniently access Azure KeyVault secrets.
    If the KeyVault cannot be accessed, it will try to read secrets from a
    .secrets file instead. This can be useful for testing or inside restricted
    environments like containers.

    Args:
        url (str): URL of the Azure KeyVault
        secrets_path (str, optional): Path to the .secrets file. Defaults to None.
    """
    def __init__(self, url, secrets_path=None):
        self.url = url
        self.secrets = dotenv.dotenv_values(dotenv_path=secrets_path)
        self.credential = get_credentials()
        self.client = SecretClient(vault_url=self.url, credential=self.credential)

    def get_secret(self, secret_name):
        secret_value = None
        try:
            secret = self.client.get_secret(secret_name)
            secret_value = secret.value
        except HttpResponseError:
            secret_value = self.secrets[secret_name]
        return Secret(secret_value)
